package org.example.wedding_app_backend.api.controller;

import org.assertj.db.api.Assertions;
import org.assertj.db.type.Column;
import org.assertj.db.type.Table;
import org.example.wedding_app_backend.WeddingAppBackendApplication;
import org.example.wedding_app_backend.initializer.DataInitializer;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.intellij.lang.annotations.Language;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.core.authority.SimpleGrantedAuthority;
import org.springframework.security.oauth2.server.resource.authentication.JwtAuthenticationToken;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.security.oauth2.jwt.Jwt;

import javax.sql.DataSource;
import java.time.Instant;
import java.time.LocalDateTime;
import java.util.List;
import java.util.UUID;

import static org.assertj.db.api.Assertions.assertThat;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.authentication;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.multipart;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.header;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.MOCK,
        classes = WeddingAppBackendApplication.class
)
@AutoConfigureMockMvc

public class WeddingControllerTests {

    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    DataSource dataSource;

    @MockBean
    private DataInitializer dataInitializer;//Disable data initialization

    @AfterEach
    void clean() {
        weddingRepository.deleteAll();
    }

    @Test
    void getWedding_ShouldReturnWedding_WhenExists() throws Exception {
        weddingRepository.save(WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build());

        @Language("JSON")
        String expected = """
                {
                  "id": "%s",
                  "name": "Wedding 1",
                  "eventStartDateTime": "2024-11-21T14:41:00"
                }
                """.formatted(UUID.nameUUIDFromBytes("wedding_1".getBytes()));

        mockMvc.perform(get("/api/weddings/{id}", UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWedding_ShouldReturnWedding_WhenNotAuthorized() throws Exception {
        weddingRepository.save(WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build());

        @Language("JSON")
        String expected = """
                {
                  "id": "%s",
                  "name": "Wedding 1",
                  "eventStartDateTime": "2024-11-21T14:41:00"
                }
                """.formatted(UUID.nameUUIDFromBytes("wedding_1".getBytes()));

        mockMvc.perform(get("/api/weddings/{id}", UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWedding_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{id}", UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void getWeddingImage_ShouldReturnWeddingBackgroundImage_WhenExists() throws Exception {
        byte[] image = new byte[]{1, 2, 3};

        weddingRepository.save(WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2025-05-13T21:00"))
                .backgroundImage(image)
                .build());

        mockMvc.perform(get("/api/weddings/{id}/background", UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                    .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.IMAGE_JPEG))
                .andExpect(content().bytes(image));
    }

    @Test
    void getWeddingImage_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{id}/background", UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }


    @Test
    @WithMockUser(username = "user", roles = {"CLIENT_USER"})
    void putWeddingBackgroundImage_ShouldSaveImage() throws Exception {
        weddingRepository.save(WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .userId(UUID.nameUUIDFromBytes("user_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build());

        MockMultipartFile mockFile = new MockMultipartFile("image", "background.jpg", "image/jpeg", new byte[]{1, 2, 3});

        Jwt jwt = Jwt.withTokenValue("mock-token")
                .header("alg", "none")
                .claim("sub", UUID.nameUUIDFromBytes("user_1".getBytes()).toString())
                .claim("scope", "openid")
                .issuedAt(Instant.now())
                .expiresAt(Instant.now().plusSeconds(3600))
                .build();

        JwtAuthenticationToken token = new JwtAuthenticationToken(
                jwt,
                List.of(new SimpleGrantedAuthority("ROLE_CLIENT_USER"))
        );

        mockMvc.perform(multipart("/api/users/background", UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                        .file(mockFile)
                        .contentType(MediaType.MULTIPART_FORM_DATA)
                        .with(authentication(token)))
                .andExpect(status().isCreated())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "weddings"))
                .hasNumberOfRows(1)
                .row()
                .column("background_image").value().isEqualTo(new byte[]{1, 2, 3});
    }
}

