package org.example.wedding_app_backend.api.controller;

import jakarta.transaction.Transactional;
import org.assertj.db.api.Assertions;
import org.assertj.db.type.Column;
import org.assertj.db.type.Table;
import org.example.wedding_app_backend.WeddingAppBackendApplication;
import org.example.wedding_app_backend.initializer.DataInitializer;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.intellij.lang.annotations.Language;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import javax.sql.DataSource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.assertj.db.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.header;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.MOCK,
        classes = WeddingAppBackendApplication.class
)
@AutoConfigureMockMvc
public class WeddingImageControllerTests {

    @Autowired
    private MockMvc mockMvc;


    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WeddingImageRepository weddingImageRepository;

    @Autowired
    private WeddingGuestRepository weddingGuestRepository;

    @MockBean
    private DataInitializer dataInitializer;//Disable data initialization

    @Autowired
    DataSource dataSource;

    @AfterEach
    void clean() {
        weddingRepository.deleteAll();
        weddingImageRepository.deleteAll();
        weddingGuestRepository.deleteAll();
    }

    @Test
    void getWeddingGallery_ShouldReturnImagesInOrder_WhenExists() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .build();
        weddingGuestRepository.save(guest_1);

        weddingImageRepository.save(WeddingImageEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                .date(LocalDateTime.parse("2025-05-13T21:00"))
                .likes(1)
                .image("image_1".getBytes())
                .wedding(wedding_1)
                .build());
        weddingImageRepository.save(WeddingImageEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_image_2".getBytes()))
                .date(LocalDateTime.parse("2025-05-13T21:01"))
                .likes(0)
                .image("image_2".getBytes())
                .wedding(wedding_1)
                .build());
        weddingImageRepository.save(WeddingImageEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_image_3".getBytes()))
                .date(LocalDateTime.parse("2025-05-13T21:02"))
                .likes(0)
                .image("image_3".getBytes())
                .wedding(wedding_1)
                .build());

        @Language("JSON")
        String expected = """
                [
                  {
                    "id": "%s",
                    "likes": 0,
                    "date": "2025-05-13T21:02",
                    "liked": false
                  },
                  {
                    "id": "%s",
                    "likes": 0,
                    "date": "2025-05-13T21:01",
                    "liked": false
                  },
                  {
                    "id": "%s",
                    "likes": 1,
                    "date": "2025-05-13T21:00",
                    "liked": false
                  }
                ]
                """.formatted(
                        UUID.nameUUIDFromBytes("wedding_image_3".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_image_2".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_image_1".getBytes())
                );

        mockMvc.perform(get("/api/weddings/{weddingId}/guests/{guestId}/images",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                        .param("sort", "date")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWeddingGallery_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{weddingId}/guests/{guestId}/images",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_not_existing".getBytes()))
                        .param("sort", "date")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void getWeddingImage_ShouldReturnWeddingImage_WhenExists() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        weddingImageRepository.save(WeddingImageEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                .date(LocalDateTime.parse("2025-05-13T21:00"))
                .likes(1)
                .image("image_1".getBytes())
                .wedding(wedding_1)
                .build());

        mockMvc.perform(get("/api/weddings/{weddingId}/images/{imageId}",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.IMAGE_JPEG))
                .andExpect(content().bytes("image_1".getBytes()));
    }

    @Test
    void getWeddingImage_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{weddingId}/images/{imageId}",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_image_not_existing".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void postWeddingImage_ShouldSaveImage() throws Exception {
        weddingRepository.save(WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build());

        MockMultipartFile mockFile = new MockMultipartFile("image", "image.jpg", "image/jpeg", new byte[]{1, 2, 3});

        mockMvc.perform(multipart("/api/weddings/{weddingId}/images/{imageId}", UUID.nameUUIDFromBytes("wedding_1".getBytes()), UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                        .file(mockFile)
                        .contentType(MediaType.MULTIPART_FORM_DATA))
                .andExpect(status().isCreated())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_images"))
                .hasNumberOfRows(1)
                .row()
                .column("image").value().isEqualTo(new byte[]{1, 2, 3});
    }

    @Test
    void changeWeddingImageLikes_ShouldAddLikes_WhenGuestDontLikesIt() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .likedImages(new ArrayList<>())
                .build();
        weddingGuestRepository.save(guest_1);

        WeddingImageEntity image_1 = WeddingImageEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                .date(LocalDateTime.parse("2025-05-13T21:00"))
                .likes(0)
                .image("image_1".getBytes())
                .wedding(wedding_1)
                .likedByGuest(new ArrayList<>())
                .build();
        weddingImageRepository.save(image_1);

        mockMvc.perform(patch("/api/weddings/{weddingId}/guests/{guestId}/images/{imageId}",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                        .param("liked", "true")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_images"))
                .row()
                .column("likes").value().isEqualTo(1);

        // Usuniecie relacji @ManyToMany żeby nie było błędu bazy
        weddingGuestRepository.save(guest_1);
        weddingImageRepository.save(image_1);
    }

    @Test
    void changeWeddingImageLikes_ShouldNotAddLikes_WhenGuestLikesIt() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingImageEntity image_1 = WeddingImageEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                .date(LocalDateTime.parse("2025-05-13T21:00"))
                .likes(1)
                .image("image_1".getBytes())
                .wedding(wedding_1)
                .likedByGuest(new ArrayList<>())
                .build();
        weddingImageRepository.save(image_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .likedImages(new ArrayList<>())
                .build();
        guest_1.getLikedImages().add(image_1);
        weddingGuestRepository.save(guest_1);

        mockMvc.perform(patch("/api/weddings/{weddingId}/guests/{guestId}/images/{imageId}",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_image_1".getBytes()))
                        .param("liked", "true")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_images"))
                .row()
                .column("likes").value().isEqualTo(1);

        // Usuniecie relacji @ManyToMany żeby nie było błędu bazy
        guest_1.setLikedImages(new ArrayList<>());
        weddingGuestRepository.save(guest_1);
        weddingImageRepository.save(image_1);
    }
}

