package org.example.wedding_app_backend.api.controller;

import jakarta.transaction.Transactional;
import org.assertj.db.api.Assertions;
import org.assertj.db.type.Column;
import org.assertj.db.type.Table;
import org.example.wedding_app_backend.WeddingAppBackendApplication;
import org.example.wedding_app_backend.initializer.DataInitializer;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.weddingImage.entity.WeddingImageEntity;
import org.example.wedding_app_backend.weddingImage.repository.WeddingImageRepository;
import org.example.wedding_app_backend.weddingQuiz.entity.WeddingQuizEntity;
import org.example.wedding_app_backend.weddingQuiz.repository.WeddingQuizRepository;
import org.intellij.lang.annotations.Language;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;

import javax.sql.DataSource;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.UUID;

import static org.assertj.db.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.header;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.MOCK,
        classes = WeddingAppBackendApplication.class
)
@AutoConfigureMockMvc
public class WeddingQuizControllerTests {

    @Autowired
    private MockMvc mockMvc;


    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WeddingQuizRepository weddingQuizRepository;

    @Autowired
    private WeddingGuestRepository weddingGuestRepository;

    @MockBean
    private DataInitializer dataInitializer;//Disable data initialization

    @Autowired
    DataSource dataSource;

    @AfterEach
    void clean() {
        weddingQuizRepository.deleteAll();
        weddingGuestRepository.deleteAll();
        weddingRepository.deleteAll();
    }

    @Test
    void getWeddingQuestion_ShouldReturnQuestions_WhenExists() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .questionIndex(0)
                .quizSeed(1)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        weddingQuizRepository.save(WeddingQuizEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_quiz_question_1".getBytes()))
                .question("Test")
                .answer1("Yes")
                .answer2("No")
                .answer3("Maybe")
                .answer4("Never")
                .wedding(wedding_1)
                .build());

        @Language("JSON")
        String expected = """
                  {
                    "question": "Test",
                    "answer1": "Yes",
                    "answer2": "No",
                    "answer3": "Maybe",
                    "answer4": "Never"
                  }
                """;

        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/games/quiz",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, true));
    }

    @Test
    void getWeddingQuestion_ShouldReturnNotFound_WhenDoesNotExist() throws Exception {
        mockMvc.perform(get("/api/weddings/{weddingId}/{guestId}/games/quiz",
                        UUID.nameUUIDFromBytes("wedding_not_existing".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_not_existing".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound())
                .andExpect(content().string(""));
    }

    @Test
    void updateWeddingGuestPoints_ShouldIncreasePointsAndIndex() throws Exception {
        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .questionIndex(0)
                .quizSeed(1)
                .quizPoints(0)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        mockMvc.perform(patch("/api/weddings/{weddingId}/{guestId}/games/quiz",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                        .content("{\"score\": 1}")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_guests"))
                .row()
                .column("quiz_Points").value().isEqualTo(1);

        assertThat(new Table(dataSource, "wedding_guests"))
                .row()
                .column("question_Index").value().isEqualTo(1);

        // Usuniecie relacji @ManyToMany żeby nie było błędu bazy
        weddingGuestRepository.save(guest_1);
    }


}

