package org.example.wedding_app_backend.api.controller;


import org.example.wedding_app_backend.WeddingAppBackendApplication;
import org.example.wedding_app_backend.initializer.DataInitializer;
import org.example.wedding_app_backend.wedding.entity.WeddingEntity;
import org.example.wedding_app_backend.wedding.repository.WeddingRepository;
import org.example.wedding_app_backend.weddingGuest.entity.WeddingGuestEntity;
import org.example.wedding_app_backend.weddingGuest.repository.WeddingGuestRepository;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.model.WhoKnowsBetterQuiz;
import org.example.wedding_app_backend.whoKnowsBetterQuiz.repository.WhoKnowsBetterQuizRepository;
import org.assertj.db.type.Table;
import org.intellij.lang.annotations.Language;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.messaging.simp.SimpMessagingTemplate;
import org.springframework.test.web.servlet.MockMvc;

import javax.sql.DataSource;
import java.time.LocalDateTime;
import java.util.UUID;

import static org.assertj.db.api.Assertions.assertThat;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.patch;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.content;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.status;

@SpringBootTest(
        webEnvironment = SpringBootTest.WebEnvironment.MOCK,
        classes = WeddingAppBackendApplication.class
)
@AutoConfigureMockMvc
public class WeddingWhoKnowsBetterQuizControllerTests {
    @Autowired
    private MockMvc mockMvc;

    @Autowired
    private WeddingRepository weddingRepository;

    @Autowired
    private WhoKnowsBetterQuizRepository whoKnowsBetterQuizRepository;

    @Autowired
    private WeddingGuestRepository weddingGuestRepository;

    @MockBean
    private DataInitializer dataInitializer;

    //do sockecikow
    @MockBean
    private SimpMessagingTemplate messagingTemplate;

    @Autowired
    DataSource dataSource;

    @AfterEach
    void clean() {
        whoKnowsBetterQuizRepository.deleteAll();
        weddingGuestRepository.deleteAll();
        weddingRepository.deleteAll();
    }

    @Test
    void getWhoKnowsBetterQuestion_ShouldReturnQuestion_WhenExists() throws Exception {

        WeddingEntity wedding_1 = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding_1);

        WeddingGuestEntity guest_1 = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .whoKnowsQuestionIndex(0)
                .wedding(wedding_1)
                .build();
        weddingGuestRepository.save(guest_1);

        WhoKnowsBetterQuiz quiz = WhoKnowsBetterQuiz.builder()
                .id(UUID.nameUUIDFromBytes("test_quiz".getBytes()))
                .question("Who is more likely to stay up late?")
                .wedding(wedding_1)
                .groomScore(2)
                .brideScore(3)
                .build();
        whoKnowsBetterQuizRepository.save(quiz);

        @Language("JSON")
        String expected = """
            {
              "id": "%s",
              "question": "Who is more likely to stay up late?",
              "groomScore": 2,
              "brideScore": 3
            }
            """.formatted(UUID.nameUUIDFromBytes("test_quiz".getBytes()));

        mockMvc.perform(get("/api/wedding/{weddingId}/{guestId}/games/who-knows-better",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().contentType(MediaType.APPLICATION_JSON))
                .andExpect(content().json(expected, false));
    }

    @Test
    void getWhoKnowsBetterQuestion_ShouldReturnNotFound_WhenNotExists() throws Exception {

        mockMvc.perform(get("/api/wedding/{weddingId}/{guestId}/games/who-knows-better",
                        UUID.nameUUIDFromBytes("nonexistent_wedding".getBytes()),
                        UUID.nameUUIDFromBytes("nonexistent_guest".getBytes()))
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isNotFound());

    }

    @Test
    void updateWhoKnowsBetterQuestionCounter_ShouldUpdateScoreAndIncrementIndex() throws Exception {
        // Create test data
        WeddingEntity wedding = WeddingEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_1".getBytes()))
                .name("Wedding 1")
                .eventStartDateTime(LocalDateTime.parse("2024-11-21T14:41"))
                .build();
        weddingRepository.save(wedding);

        WeddingGuestEntity guest = WeddingGuestEntity.builder()
                .id(UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                .whoKnowsQuestionIndex(0)
                .wedding(wedding)
                .build();
        weddingGuestRepository.save(guest);

        WhoKnowsBetterQuiz quiz = WhoKnowsBetterQuiz.builder()
                .id(UUID.nameUUIDFromBytes("wedding_quiz_1".getBytes()))
                .question("Who is more likely to be late?")
                .groomScore(2)
                .brideScore(3)
                .wedding(wedding)
                .build();
        whoKnowsBetterQuizRepository.save(quiz);

        // Test the endpoint with bride answer
        mockMvc.perform(patch("/api/wedding/{weddingId}/{guestId}/games/who-knows-better/answer",
                        UUID.nameUUIDFromBytes("wedding_1".getBytes()),
                        UUID.nameUUIDFromBytes("wedding_guest_1".getBytes()))
                        .content("{\"answer\": \"bride\"}")
                        .contentType(MediaType.APPLICATION_JSON))
                .andExpect(status().isOk())
                .andExpect(content().string(""));

        assertThat(new Table(dataSource, "wedding_who_knows_better_quiz_scores"))
                .row()
                .column("bride_score").value().isEqualTo(4);

        // Verify the groom score was not changed
        assertThat(new Table(dataSource, "wedding_who_knows_better_quiz_scores"))
                .row()
                .column("groom_score").value().isEqualTo(2);

        assertThat(new Table(dataSource, "wedding_guests"))
                .row()
                .column("who_knows_question_index").value().isEqualTo(1);
    }

}
