import {
  Component,
  Input,
  OnInit,
} from '@angular/core';
import { GalleryService } from '../../services/gallery.service';
import {NgClass, NgForOf} from '@angular/common';
import { CascadeSelectModule } from 'primeng/cascadeselect';
import { PanelModule } from 'primeng/panel';
import {FileUploadModule} from 'primeng/fileupload';
import { ButtonGroupModule } from 'primeng/buttongroup';
import {ImageType} from '../../models/image.type';
import {ConfirmationService, MenuItem, MessageService} from 'primeng/api';
import {TabMenuModule} from 'primeng/tabmenu';
import {DomSanitizer, SafeUrl} from '@angular/platform-browser';
import {DataViewModule} from 'primeng/dataview';
import {ToastModule} from 'primeng/toast';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {DialogModule} from 'primeng/dialog';
import {SecuredImagePipe} from '../../pipes/secured-image.pipe';

interface PayloadMenuItem {
  label: string;
  payload: string;
}

@Component({
  selector: 'app-gallery',
  standalone: true,
  imports: [
    NgForOf,
    CascadeSelectModule,
    PanelModule,
    FileUploadModule,
    ButtonGroupModule,
    TabMenuModule,
    DataViewModule,
    NgClass,
    ToastModule,
    ConfirmDialogModule,
    DialogModule,
    SecuredImagePipe
  ],
  providers: [ConfirmationService, MessageService],
  templateUrl: './gallery.component.html',
  styleUrls: ['./gallery.component.css']
})
export class GalleryComponent {

  gallery: ImageType[] = [];

  displayDialog: boolean = false;

  @Input()
  weddingId: string = '';

  @Input()
  guestId: string = '';

  sorts: Array<PayloadMenuItem> = [
    {
      label: 'Najnowsze',
      payload: 'date'
    },
    {
      label: 'Popularne',
      payload: 'likes'
    }
  ];

  activeSort: PayloadMenuItem = this.sorts[0];

  constructor(private galleryService: GalleryService,
              private messageService: MessageService) {
  }

  showDialog() {
    this.displayDialog = true;
  }

  onUpload(event: any) {
    if (!event.files || event.files.length === 0) {
      this.messageService.add({
        severity: 'warn',
        summary: 'Brak plików',
        detail: 'Nie wybrano żadnych plików do przesłania.'
      });
      return;
    }

    for (let file of event.files) {
      this.galleryService.postImage(file, this.weddingId, this.guestId).subscribe({
        next: () => {
          this.messageService.add({
            severity: 'success',
            summary: 'Plik przesłany',
            detail: `Plik ${file.name} został przesłany.`
          });
        },
        error: (err) => {
          this.messageService.add({
            severity: 'error',
            summary: 'Błąd przesyłania',
            detail: `Plik ${file.name} nie został przesłany.`
          });
        }
      });

      this.displayDialog = false;
    }

    this.loadGallery();
  }

  onDownloadImageAction(imageId: string): void {
    const url = `/api/wedding/${this.weddingId}/gallery/${imageId}`;
    const link = document.createElement('a');
    link.href = url;
    link.download = `${imageId}.jpg`;
    link.style.display = 'none';

    document.body.appendChild(link);
    link.click();
    document.body.removeChild(link);
  }

  onSortTypeChanged(sort: MenuItem): void {
    this.activeSort = sort as PayloadMenuItem;
    this.loadGallery();
  }

  onLikeButtonAction(image: ImageType): void {
    image.liked = !image.liked;

    if (image.liked) {
      image.likes = image.likes + 1;
    }
    else {
      image.likes = image.likes - 1;
    }

    this.galleryService.changeImageLikes(this.weddingId, this.guestId, image.id, image.liked).subscribe({
      error: (error) => console.error('Error updating like:', error)
    });
  }

  loadGallery(): void {
    this.galleryService
      .getGalleryByWeddingId(this.weddingId, this.guestId, this.activeSort.payload)
      .subscribe({
        next: (weddingGallery) => {
          this.gallery = weddingGallery;
        },
        error: (error) => console.error('Error fetching gallery:', error)
      });
  }

}

