import {Component, ElementRef, OnInit, ViewChild} from '@angular/core';
import {catchError} from 'rxjs';
import {ConfirmationGroupType} from '../../models/confirmationGroup.type';
import {GuestsService} from '../../services/guests.service';
import {GuestType} from '../../models/guests.type';
import {Button} from 'primeng/button';
import {TableModule} from 'primeng/table';
import {PaginatorModule} from 'primeng/paginator';
import {CommonModule} from '@angular/common';
import {TagModule} from 'primeng/tag';
import {InputTextModule} from 'primeng/inputtext';
import {InputIconModule} from 'primeng/inputicon';
import {IconFieldModule} from 'primeng/iconfield';
import {CardModule} from 'primeng/card';
import {DividerModule} from 'primeng/divider';
import {ConfirmationService, MessageService} from 'primeng/api';
import {FilterPipe} from '../../pipes/filter.pipe';
import {ToolbarModule} from 'primeng/toolbar';
import {ConfirmDialogModule} from 'primeng/confirmdialog';
import {MessagesModule} from 'primeng/messages';
import {DialogModule} from 'primeng/dialog';
import {CheckboxModule} from 'primeng/checkbox';
import {RadioButtonModule} from 'primeng/radiobutton';
import {AutoCompleteModule} from 'primeng/autocomplete';
import {MenuModule} from 'primeng/menu';
import {KeycloakService} from '../../keycloak/keycloak.service';
import {ToastModule} from 'primeng/toast';
import {Ripple} from 'primeng/ripple';
import {SelectButtonModule} from 'primeng/selectbutton';

@Component({
  selector: 'app-guests',
  standalone: true,
  imports: [
    Button,
    TableModule,
    PaginatorModule,
    TagModule,
    InputTextModule,
    IconFieldModule,
    InputIconModule,
    CardModule,
    DividerModule,
    FilterPipe,
    CommonModule,
    ToolbarModule,
    ConfirmDialogModule,
    MessagesModule,
    DialogModule,
    CheckboxModule,
    RadioButtonModule,
    AutoCompleteModule,
    MenuModule,
    ToastModule,
    Ripple,
    SelectButtonModule,
  ],
  providers: [MessageService, ConfirmationService],
  templateUrl: './guests.component.html',
  styleUrl: './guests.component.css',
  host: { ngSkipHydration: 'true' },
})
export class GuestsComponent implements OnInit {

  weddingGuests: GuestType[] = [];

  weddingId: string = '';
  guestId:string = '';

  searchText:any;

  menuAge: any[] = [
    { name: 'Dorośli', value: 'Dorosły' },
    { name: 'Młodzież', value: 'Młodzież' },
    { name: 'Dzieci', value: 'Dziecko' }
  ];

  filteredGuestsNames: any[] = [];

  groupDialog:boolean = false;
  guestNumber:number=0;

  guestDialog: boolean = false;
  submitted: boolean = false;

  guestForm: GuestType={
    id:'',
    guestName:'',
    presenceConfirmed:null,
    guestAge:'brak',
    weddingConfirmationGroup:{id:'',name:''}
  };

  groupForm: ConfirmationGroupType={
    id:'',
    name:'',
  };

  guestsGroupTable:GuestType[]=[];

  constructor(private GuestService: GuestsService,
              private confirmationService: ConfirmationService,
              private messageService: MessageService,
              private keycloakService: KeycloakService
  ) {}


  ngOnInit(): void {
    if(this.keycloakService.getToken()){
      this.getGuests();
    }
  }

  getGuests() {
    this.GuestService.getGuestsByWeddingId().pipe(
      catchError((error) => {
        console.error('Error fetching wedding guests:', error);
        throw error;
      })
    ).subscribe((weddingGuests) => {
      this.weddingGuests = weddingGuests;
    });

  }

  //core functions

  addGuest(group:ConfirmationGroupType) {
    this.resetGuestForm()
    this.guestForm.weddingConfirmationGroup=group;
    this.submitted = false;
    this.guestDialog = true;
  }

  addGroup(){
    this.groupForm={id:'',name:''};
    this.submitted = false;
    this.groupDialog = true;
  }

  editGuest(guest: GuestType) {
    this.guestForm = { ...guest };
    this.guestDialog = true;
  }

  deleteGuest(guest: GuestType) {
    this.confirmationService.confirm({
      message: 'Czy na pewno chcesz usunąć tego gościa?',
      header: 'Potwierdzenie',
      acceptLabel: 'Tak, usuń',
      rejectLabel: 'Anuluj',
      accept: () => {
        this.GuestService.deleteGuestInformation(guest).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Gość usunięty',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd!',
              detail: 'Nie udało się usunąć gościa',
              life: 3000
            });
          }
        });
      }
    });
  }

  deleteGroup(group: ConfirmationGroupType) {
    this.confirmationService.confirm({
      message: 'Czy na pewno chcesz usunąć tą grupę?',
      header: 'Potwierdzenie',
      acceptLabel: 'Tak, usuń',
      rejectLabel: 'Anuluj',
      accept: () => {
        this.GuestService.deleteGroup(group).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Grupa usunięta',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd!',
              detail: 'Nie udało się usunąć grupy',
              life: 3000
            });
          }
        });
      }
    });
  }

  saveGuest() {
    this.submitted = true;

    if (this.guestForm.guestName?.trim()) {
      if (this.guestForm.id) {
        this.GuestService.patchGuestInformation(this.guestForm).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Dane zminenione',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd',
              detail: 'Nie udało się zmienić danych',
              life: 3000
            });
          }
        });
      } else {
        this.GuestService.postGuestInformation(this.guestForm).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Gość dodany',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd',
              detail: 'Nie udało się utworzyć gościa',
              life: 3000
            });
          }
        });
      }

      this.guestDialog = false;
      this.resetGuestForm();
    }
  }

  saveGroup() {
    this.submitted = true;

    if (this.groupForm.name?.trim() && this.guestsGroupTable.every(guest => guest.guestName?.trim())) {
      if (this.guestsGroupTable.length > 0) {
        this.guestsGroupTable.forEach(guest => {guest.weddingConfirmationGroup.name=this.groupForm.name})
        this.GuestService.createGroup(this.guestsGroupTable).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Dane zminenione',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd',
              detail: 'Nie udało się zmienić danych',
              life: 3000
            });
          }
        });
      } else {
        this.guestForm.weddingConfirmationGroup.name=this.groupForm.name
        this.guestsGroupTable.push(this.guestForm)
        this.GuestService.createGroup(this.guestsGroupTable).subscribe({
          next: () => {
            this.getGuests();
            this.messageService.add({
              severity: 'success',
              summary: 'Udało się!',
              detail: 'Gość dodany',
              life: 3000
            });
          },
          error: () => {
            this.messageService.add({
              severity: 'error',
              summary: 'Błąd',
              detail: 'Nie udało się utworzyć gościa',
              life: 3000
            });
          }
        });
      }

      this.guestsGroupTable=[]
      this.groupForm={
        id:'',
        name:'',
      }
      this.guestDialog = false;
    }
  }

  hideDialog() {
    this.guestDialog = false;
    this.submitted = false;
    this.groupDialog = false;
    this.guestNumber=0;
    this.guestsGroupTable=[];
  }

  //helping functions

  getPresenceDisplay(isConfirmed: boolean): string {
    return isConfirmed ? 'Tak' : 'Nie';
  }

  getGuestsCountByAccept(): number {
    return this.weddingGuests.filter(guest => guest.presenceConfirmed).length;
  }

  getSeverity(status: string) {
    switch (status) {
      case "Dorosły":
        return 'success';
      case "Młodzież":
        return 'info';
      case "Dziecko":
        return 'warning';
      case "Tak":
        return 'success';
      case "Nie":
        return 'danger';
      default:
        return 'info';
    }
  }

  filterGuestsNames(event: any) {
    const query = event.query.toLowerCase();
    this.filteredGuestsNames = Array.from(
      new Set(
        this.weddingGuests
          .filter((guest) =>
            guest.guestName.toLowerCase().startsWith(query)
          )
          .map((guest: GuestType) => guest.guestName)
      )
    );
  }

  filterPresence() {
    this.searchText = this.searchText === 'true' ? '' : 'true';
  }

  resetGuestForm(){
    this.guestForm = {guestAge: 'Brak', presenceConfirmed: null, guestName:'',id:'',weddingConfirmationGroup:{id:'',name:''}};
  }

  calculateGuestGroupTotal(group:ConfirmationGroupType) {
    let total = 0;

    if (this.weddingGuests) {
      for (let guest of this.weddingGuests) {
        if (guest.weddingConfirmationGroup?.name === group.name && guest.weddingConfirmationGroup?.id === group.id) {
          total++;
        }
      }
    }

    return total;
  }

  decreaseGuest() {
    if (this.guestNumber > 0) {
      this.guestNumber--;
    }
    if (this.guestsGroupTable.length > 0) {
      this.guestsGroupTable.pop();
    }
  }

  increaseGuest() {
    if(this.guestNumber < 4) {
      this.guestNumber++;
      this.guestsGroupTable.push({
        id: '',
        guestName: '',
        presenceConfirmed: null,
        guestAge: 'brak',
        weddingConfirmationGroup: { id: '', name: '' }
      })
    }
  }

}
