import { ComponentFixture, TestBed } from '@angular/core/testing';
import { QuizComponent } from './quiz.component';
import { QuizService } from '../../services/quiz.service';
import { Router } from '@angular/router';
import { ActivatedRoute } from '@angular/router';
import { of } from 'rxjs';

describe('QuizComponent', () => {
  let component: QuizComponent;

  const mockQuizService = {
    getQuizStateByWeddingId: jasmine.createSpy('getQuizStateByWeddingId').and.returnValue(of({ quizFinished: false })),
    getQuizByWeddingId: jasmine.createSpy('getQuizByWeddingId').and.returnValue(of({
      question: 'Sample?',
      answer1: 'A',
      answer2: 'B',
      answer3: 'C',
      answer4: 'D',
    })),
    getQuizScoreByWeddingId: jasmine.createSpy('getQuizScoreByWeddingId').and.returnValue(of({
      questionIndex: 0,
      quizPoints: 0,
      guestName: '',
    })),
    sendPointsByGuestId: jasmine.createSpy('sendPointsByGuestId').and.returnValue(of({}))
  };

  const mockActivatedRoute = {
    snapshot: {
      paramMap: {
        get: (key: string) => {
          if (key === 'weddingId') return '123';
          if (key === 'guestId') return '456';
          return null;
        }
      }
    }
  };

  const mockRouter = {
    navigateByUrl: jasmine.createSpy('navigateByUrl')
  };

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      imports: [QuizComponent],
      providers: [
        { provide: QuizService, useValue: mockQuizService },
        { provide: ActivatedRoute, useValue: mockActivatedRoute },
        { provide: Router, useValue: mockRouter }
      ]
    }).compileComponents();

    const fixture = TestBed.createComponent(QuizComponent);
    component = fixture.componentInstance;
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  it('should initialize weddingId and guestId in ngOnInit', () => {
    component.ngOnInit();
    expect(component.weddingId).toBe('123');
    expect(component.guestId).toBe('456');
  });

  it('should fetch quiz state and get question if quiz is not finished', () => {
    component.ngOnInit();
    component.getQuestion();
    expect(mockQuizService.getQuizStateByWeddingId).toHaveBeenCalledWith('123', '456');
    expect(mockQuizService.getQuizByWeddingId).toHaveBeenCalledWith('123', '456');
  });

  it('should fetch quiz score if quiz is finished', () => {
    component.ngOnInit();
    mockQuizService.getQuizStateByWeddingId.and.returnValue(of({ quizFinished: true }));
    component.getQuestion();
    expect(mockQuizService.getQuizScoreByWeddingId).toHaveBeenCalledWith('123', '456');
  });

  it('should fetch next question and shuffle answers', () => {
    component.ngOnInit();
    spyOn(component, 'shuffleAnswers');
    component.nextQuestion();
    expect(mockQuizService.getQuizByWeddingId).toHaveBeenCalledWith('123', '456');
    expect(component.question.question).toBe('Sample?');
    expect(component.shuffleAnswers).toHaveBeenCalled();
  });

  it('should fetch quiz score', () => {
    component.ngOnInit();
    component.getScore();
    expect(mockQuizService.getQuizScoreByWeddingId).toHaveBeenCalledWith('123', '456');
  });

  it('should navigate to hall of fame on onClickRoute', () => {
    component.ngOnInit();
    component.onClickRoute();
    expect(mockRouter.navigateByUrl).toHaveBeenCalledWith('/wedding/123/456/games/hall');
  });

  it('should shuffle answers correctly', () => {
    component.ngOnInit();
    component.question = {
      question: 'Sample?',
      answer1: 'A',
      answer2: 'B',
      answer3: 'C',
      answer4: 'D',
    };

    spyOn(Math, 'random').and.returnValues(0.3, 0.6, 0.9, 0.1);
    component.shuffleAnswers();

    expect(component.shuffledAnswers.length).toBe(4);
    expect(component.shuffledAnswers).toContain('A');
    expect(component.shuffledAnswers).toContain('B');
    expect(component.shuffledAnswers).toContain('C');
    expect(component.shuffledAnswers).toContain('D');
  });

  it('should process answer selection and send points', () => {
    component.ngOnInit();
    component.question = {
      question: 'Sample?',
      answer1: 'A',
      answer2: 'B',
      answer3: 'C',
      answer4: 'D',
    };

    component.onAnswerSelected('A'); // Correct answer
    expect(component.correct).toBeTrue();
    expect(mockQuizService.sendPointsByGuestId).toHaveBeenCalledWith('123', '456', 1);

    component.onAnswerSelected('B'); // Incorrect answer
    expect(component.correct).toBeFalse();
    expect(mockQuizService.sendPointsByGuestId).toHaveBeenCalledWith('123', '456', 0);
  });
});
