import { ChangeDetectorRef, OnDestroy, Pipe, PipeTransform } from '@angular/core';
import { BehaviorSubject, distinctUntilChanged, map, Observable, Subscription, switchMap, tap } from 'rxjs';
import { HttpClient } from '@angular/common/http';
import { DomSanitizer, SafeUrl } from '@angular/platform-browser';
import { filter } from "rxjs/operators";

@Pipe({
  name: 'securedImage',
  standalone: true,
  pure: false
})
export class SecuredImagePipe implements PipeTransform, OnDestroy {

  private subscription = new Subscription();

  private transformValue = new BehaviorSubject<string>('');

  private latestValue!: string | SafeUrl;

  constructor(
    private http: HttpClient,
    private sanitizer: DomSanitizer,
    private cdr: ChangeDetectorRef
  ) {
    this.initSubscription();
  }

  transform(url: string): any {
    this.transformValue.next(url);
    return this.latestValue;
  }

  ngOnDestroy(): void {
    this.subscription.unsubscribe();
  }

  private initSubscription() {
    const transformSubscription = this.transformValue
      .asObservable()
      .pipe(
        filter(v => !!v),
        distinctUntilChanged(),
        switchMap(path => this.http
          .get(path, {observe: 'response', responseType: 'blob'})
          .pipe(
            filter(response => !!response.body),
            map(response => URL.createObjectURL(response.body as Blob)),
            map(blobUrl => this.sanitizer.bypassSecurityTrustUrl(blobUrl)),
            filter(blobUrl => blobUrl !== this.latestValue)
          )
        ),
        tap((path: string | SafeUrl) => {
          this.latestValue = path;
          this.cdr.markForCheck();
        })
      )
      .subscribe();
    this.subscription.add(transformSubscription);
  }
}
